import pytest

from sequence_labeling.laso_tagger import WeakilySupervisedPerceptron


@pytest.fixture
def perceptron():
    weights = {"A": {"α": 10, "β": 1, "γ": 1},
               "B": {"α": 1, "β": 10, "γ": 1},
               "C": {"α": 1, "β": 1, "γ": 1}}

    p = WeakilySupervisedPerceptron()
    p.weights = weights

    return p


def test_single_update_one_label(perceptron):

    perceptron.update({"A"}, "C", {"α": 1, "β": 1, "γ": 1})

    expected_weights = {"A": {"α": 11, "β": 2, "γ": 2},
                        "B": {"α": 1, "β": 10, "γ": 1},
                        "C": {"α": 0, "β": 0, "γ": 0}}

    assert perceptron.weights == expected_weights


def test_single_update_several_labels(perceptron):

    perceptron.update({"A", "B"}, "C", {"α": 1, "β": 1, "γ": 1})

    expected_weights = {"A": {"α": 11, "β": 2, "γ": 2},
                        "B": {"α": 2, "β": 11, "γ": 2},
                        "C": {"α": 0, "β": 0, "γ": 0}}

    assert perceptron.weights == expected_weights


def test_predict(perceptron):

    features = {"α": 1, "γ": 1}

    assert perceptron.predict(features) == "A"
