# Copyright (c) 2014, G. Wisniewski
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:

# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.

# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.

# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
"""
Extract alignment constraints from a corpus

These constraints are use to complemente the type constraints extracted from
wiktionary.
"""
from collections import defaultdict, Counter
import pickle

from weakly_io import pickle_reader


def extract_alignment_constraints(corpus_fn, output_fn, max_ex, listener):
    """
    Extract alignment constraints from the pickled corpus stored in
    `corpus_fn`, considering only the first `max_ex` examples. The
    result is dumped to the json file `output_fn`.
    """

    corpus = open(corpus_fn, "rb")
    output = open(output_fn, "wb")
    al_constraints = defaultdict(Counter)

    for example in listener.iter(pickle_reader(corpus, max_ex)):

        for src_indice, tgt_indices in example["al"].items():

            # We are only considering 1:1 mappings
            if len(tgt_indices) != 1:
                continue

            for tgt_indice in tgt_indices:

                src_pos = example["src_pos"][src_indice]
                tgt_word = example["tgt"][tgt_indice]

                tgt_word = tgt_word.strip().lower()

                al_constraints[tgt_word].update([src_pos])

    l = len(al_constraints)
    listener.msg("extracted tag information for {:,} types".format(l))
    pickle.dump(al_constraints, output, pickle.HIGHEST_PROTOCOL)
